/********************************
* Software IIC                  *
* soft_iic.c                    *
* v1.1 07/01/2000               *
* David Procter                 *
* Senior Applications Engineer  *
* Mitsubishi Electric Europe    *
********************************/

#include "main.h"
#include "soft_iic.h"

unsigned char temp_cnt;

/**********************************************************************
* Read in a byte, send the Acknowledge bit (ack) and return the byte. *
**********************************************************************/
unsigned char iic_byte_rx(unsigned char ack){
	unsigned char bit_ptr = MSb, tbyte = 0x00;
	unsigned int timeout;
	do{
		SCL_HIZ;                    /* Release SCL.  SCL als IN-Port */
		if (SCL == LOW){            /* Slave stretching the clock ? */
			timeout = 0xFFFF;
			while (SCL == LOW) 
			    if (--timeout == 0) 
			        iic_error(RX_SCL_STUCK_LOW);
		}
		if (SDA != LOW) 
		    tbyte += bit_ptr;       /* Store the value of SDA. */
		bit_ptr >>= 1;              /* Move the pointer one to the right. */
		PAUSE;
		SCL_LOW;                    /* Pull SCL low. */
		PAUSE;
	}while (bit_ptr != 0);          /* All 8 bits read ? */

	if (ack == ACK){
		SDA_LOW;
	}else{
		SDA_HIZ;                    /* Acknowledge bit. */
	}

	SCL_HIZ;                        /* Send */
	PAUSE;
	SCL_LOW;                        /* it. */
	PAUSE;
	SDA_HIZ;                        /* Release SDA. */

	return (tbyte);                 /* Return the received byte. */
}

/*****************************************************************
* Clock out 8 data bits, read the Acknowledge bit and return it. *
*****************************************************************/
unsigned char iic_byte_tx(unsigned char tbyte){
	unsigned char bit_ptr = 7, ack;
	unsigned int timeout;
	do{
		if ((tbyte & MSb) == 0){
			SDA_LOW;
		}else{
			SDA_HIZ; 
		}
		SCL_HIZ;                    /* Release SCL. */
		if (SCL == LOW){            /* Slave stretching the clock ? */
			timeout = 0xFFFF;
			while (SCL == LOW) 
			    if (--timeout == 0) 
			        iic_error(TX_SCL_STUCK_LOW);
		}
		tbyte <<= 1;                /* Prepare the next bit. */
		PAUSE;
		SCL_LOW;                    /* Pull SCL low. */
		PAUSE;
	}while (bit_ptr-- != 0);        /* All bits sent ? */
	SDA_HIZ;                        /* Release SDA. */
	PAUSE;
	SCL_HIZ;                        /* Read the */
	PAUSE;
	ack = SDA;                      /* Acknowledge */
	SCL_LOW;                        /* bit. */
	return (ack);                   /* Return the Acknowledge bit. */
}

/****************************************************************
* If we arrive here then a non-recoverable error has occurred   *
* whereby a slave device has locked the bus by holding SCL low. *
****************************************************************/
void iic_error(unsigned char error_message){
	unsigned char diagnostic = error_message;
	for(;;);
}

/*****************************************************************
* Generate the Start sequence (SDA goes low while SCL is high).  *
* Also used for Repeat-starts.                                   *
*****************************************************************/
void iic_start(void){
	unsigned int timeout;
	SCL_HIZ;                        /* Release SCL */
	SDA_HIZ;                        /* and SDA. */
	if ((SCL == LOW) || (SDA == LOW)){/* Have they gone high ? */
		timeout = 0xFFFF;
		while ((SCL == LOW) || (SDA == LOW)) 
		    if (--timeout == 0) 
		        iic_error(START_STUCK_LOW);
	}
	SDA_LOW;                        /* Pull SDA low. */
	PAUSE;
	SCL_LOW;                        /* Pull SCL low. */
    PAUSE;
}

/****************************************************************
* Generate the Stop sequence (SDA goes high while SCL is high). *
****************************************************************/
void iic_stop(void){
	unsigned int timeout;
	SDA_LOW;                        /* Pull SDA low. */
	PAUSE;
	SCL_HIZ;                        /* Release SCL. */
	if (SCL == LOW){                /* Has SCL gone high ? */
		timeout = 0xFFFF;
		while (SCL == LOW) 
		    if (--timeout == 0) 
		        iic_error(STOP_SCL_STUCK_LOW);
	}
	SDA_HIZ;                        /* Release SDA. */
}


/****************************************************************
*   Write a byte into an iic-eeprom                             *
*                                                               *
*	return 	1 for deveiceerror                                  *
*			2 for address error                                 *
*			3 for write error                                   *
*			4 for ack timeout                                   *
****************************************************************/
unsigned char i2c_eeprom_byte_write(unsigned char device,
                                    unsigned char address, 
                                    unsigned char value){
	char retry = 50;
	int i;
	unsigned char result = 0;
	iic_start();
	if (iic_byte_tx(device) != ACK)  result = 1;
	if (iic_byte_tx(address) != ACK) result = 2; /* Sub-address */
	if (iic_byte_tx(value) != ACK) 	 result = 3;	
	iic_stop();

	do{
        for (i= 0; i< 10;i++) PAUSE;
		iic_start();	
		if (iic_byte_tx(device) != ACK){  /* polling for ACK */
			retry --;
			if (retry == 0) result = 4; 
		}
		else
			retry = 0;
		iic_stop();	
	}while (retry > 0 );
	return result;
}

/****************************************************************
*   Read a byte from an iic-eeprom                              *
****************************************************************/
unsigned char i2c_eeprom_random_read(unsigned char device, 
                                     unsigned char address){
	unsigned char result = 0;
	unsigned char temp = device;
	temp |= 0x01;			            /* set R/W# bit to 1 */
	iic_start();	
	if (iic_byte_tx(device) != ACK) ; 	
	if (iic_byte_tx(address) != ACK) ;  /* Sub-address */	
	iic_start();
	if (iic_byte_tx(temp) != ACK) ;		
	result = iic_byte_rx(NO_ACK);
	iic_stop();
    return result;
}




